@@ -1,8 +1,16 @@
+//! This is a simple crate that makes making qwac games easier.
+//!
+//! It has one feature, `alloc`.  You should enable it if you plan on using liballoc (including if
+//! `std` is enabled), otherwise, leave it disabled to save some space.
+
 #![no_std]
 
 pub mod event;
 pub mod ffi;
 
+#[cfg(feature = "alloc")]
+pub extern crate alloc;
+
 #[derive(Default, Clone, Copy, Hash, Debug, Eq, PartialEq, Ord, PartialOrd)]
 pub struct Rectangle {
     pub x: u16,
@@ -28,38 +36,54 @@ pub struct Color {
 
 pub trait Game {
     /// Create a new Game of this type, starting at the given time in seconds.
-    fn new(timestamp: f64) -> Self;
+    fn new(timestamp: i64) -> Self;
 
     /// Handle an input event.
     fn event(&mut self, event: event::Event);
 
     /// Do processing and rendering.  delta tells you the time in seconds between this frame and
     /// the last one.
-    fn update(&mut self, delta: f32);
+    fn update(&mut self, frame_milliseconds: u32);
 }
 
 #[macro_export]
-macro_rules! wag_game {
+macro_rules! qwac_game {
     ($game_type:ty) => {
-        static mut wag_rust_game: Option<$game_type> = None;
+        #[cfg(not(feature = "alloc"))]
+        static mut QWAC_RUST_GAME: Option<$game_type> = None;
 
         #[no_mangle]
-        pub extern "C" fn wag_init(timestamp: f64) {
-            unsafe {
-                wag_rust_game = None;
-                wag_rust_game = Some(<$game_type as $crate::Game>::new(timestamp));
+        pub extern "C" fn qwac_game_create(timestamp: i64) -> *mut $game_type {
+            let game = <$game_type as $crate::Game>::new(timestamp);
+
+            #[cfg(not(feature = "alloc"))]
+            {
+                unsafe {
+                    QWAC_RUST_GAME = None;
+                    QWAC_RUST_GAME = Some(game);
+                }
+                core::ptr::null_mut()
             }
+
+            #[cfg(feature = "alloc")]
+            $crate::alloc::boxed::Box::new(game).into_raw()
         }
 
         #[no_mangle]
-        pub extern "C" fn wag_event(
+        pub extern "C" fn qwac_game_event(
+            _game: *mut $game_type,
             event_type: $crate::ffi::EventType,
             player: u8,
             button: u8,
             x: u16,
             y: u16,
         ) {
-            let game = unsafe { wag_rust_game.as_mut().unwrap() };
+            #[cfg(not(feature = "alloc"))]
+            let game = unsafe { QWAC_RUST_GAME.as_mut().unwrap() };
+
+            #[cfg(feature = "alloc")]
+            let game = unsafe { &mut *_game };
+
             match event_type {
                 $crate::ffi::EventType::GamepadButtonPress => {
                     if let Some(button) = $crate::event::GamepadButton::from_code(button) {
@@ -90,16 +114,28 @@ macro_rules! wag_game {
         }
 
         #[no_mangle]
-        pub extern "C" fn wag_update(delta: f32) {
-            let game = unsafe { wag_rust_game.as_mut().unwrap() };
-            $crate::Game::update(game, delta);
+        pub extern "C" fn qwac_game_update(_game: *mut $game_type, frame_milliseconds: u32) {
+            #[cfg(not(feature = "alloc"))]
+            let game = unsafe { QWAC_RUST_GAME.as_mut().unwrap() };
+
+            #[cfg(feature = "alloc")]
+            let game = unsafe { &mut *_game };
+
+            $crate::Game::update(game, frame_milliseconds);
         }
 
         #[no_mangle]
-        pub extern "C" fn wag_shutdown() {
+        pub extern "C" fn qwac_game_destroy(_game: *mut $game_type) {
+            #[cfg(not(feature = "alloc"))]
             unsafe {
-                wag_rust_game = None;
+                QWAC_RUST_GAME = None;
             }
+
+            #[cfg(feature = "alloc")]
+            unsafe {
+                $crate::alloc::boxed::Box::from_raw(game);
+            }
+
         }
     };
 }
@@ -108,31 +144,31 @@ pub fn log(string: &str) {
     let pointer = string.as_ptr();
     let len = string.len() as u32;
     unsafe {
-        ffi::wag_log(pointer, len);
+        ffi::qwac_log(pointer, len);
     }
 }
 
 pub fn display(width_px: u16, height_px: u16) {
     unsafe {
-        ffi::wag_display(width_px, height_px);
+        ffi::qwac_display(width_px, height_px);
     }
 }
 
 pub fn clear_color(color: OpaqueColor) {
     unsafe {
-        ffi::wag_clear_color(color.r, color.g, color.b);
+        ffi::qwac_clear_color(color.r, color.g, color.b);
     }
 }
 
 pub fn clear() {
     unsafe {
-        ffi::wag_clear();
+        ffi::qwac_clear();
     }
 }
 
 pub fn fill_rectangle(rectangle: Rectangle, color: Color) {
     unsafe {
-        ffi::wag_fill_rectangle(
+        ffi::qwac_fill_rectangle(
             rectangle.x,
             rectangle.y,
             rectangle.width,
